location_libparse := "/home/sebastian/Software/Singular-git/Singular/libparse";
singular_dir := "/home/sebastian/Software/Singular-git/Singular/";
singular_dir_in_gap := Directory( singular_dir );


LoadPackage( "AutoDoc" );
LoadPackage( "JSON" );

lib_files := AUTODOC_FindMatchingFiles( singular_dir_in_gap, [ "LIB/" ], [ "lib" ] );

BindGlobal( "FindNextIndexEntryInList",
  function( list, entry )
    local current_smallest, current_index, i;
    current_smallest := First( list, i -> i > entry and i <> fail );
    if current_smallest = fail then
        return fail;
    fi;
    current_index := Position( list, current_smallest );
    for i in [ 1 .. Length( list ) ] do
        if list[ i ] < current_smallest and list[ i ] > entry and list[ i ] <> fail then
            current_smallest := list[ i ];
            current_index := i;
        fi;
    od;
    return [ current_smallest, current_index ];
end );

extract_info_singular_lib := function( filename, libparse, filename_short )
  local file, list, line, output_funcs, func_recs, output_rec, current_function_line,
        current_func_rec, first_whitespace, function_head_line_number, function_head,
        open_par, close_par, arguments, body_begin, doc_string, i, whitespace;

    file := InputTextFile( filename );
    list := [ ];
    line := ReadLine( file );
    while IsString( line ) do
        Add( list, line );
        line := ReadLine( file );
    od;

    output_funcs := IO_Popen( "/home/sebastian/Software/Singular-git/Singular/libparse", [ filename ], "r" );
    output_funcs := IO_ReadLines( output_funcs );
    output_funcs := output_funcs{[ 4 .. Length( output_funcs ) ]};

    ##filter global ones

    func_recs := [ ];

    output_rec := rec( library_filename := filename,
                       library_filename_short := filename_short,
                       funcs := func_recs );

    for current_function_line in output_funcs do
        current_func_rec := rec();
        Add( func_recs, current_func_rec );
        if current_function_line[ 1 ] = 'g' then
            current_func_rec.is_global := true;
        else
            current_func_rec.is_global := false;
        fi;
        
        NormalizeWhitespace( current_function_line );
        
        for i in [ 1 .. 2 ] do
            first_whitespace := Position( current_function_line, ' ' );
            current_function_line := current_function_line{[ first_whitespace + 1 .. Length( current_function_line ) ]};
        od;
        
        whitespace := Position( current_function_line, ' ' );
        current_func_rec.name := current_function_line{[ 1 .. whitespace - 1 ]};
        
        current_function_line := current_function_line{[ whitespace+6 ..Length( current_function_line ) ]};
        
        function_head_line_number := Position( current_function_line, ',' );
        function_head_line_number := Int( current_function_line{[ 1 .. function_head_line_number - 1]} );
        
        function_head := ShallowCopy( list[ function_head_line_number ] );
        open_par := Position( function_head, '(' );
        if open_par = fail then
            Remove( func_recs );
#             Print( "Error, function ", current_func_rec.name, " has no arguments\n" );
            current_func_rec.arguments := [ ];
            continue;
        fi;
        close_par := Position( function_head, ')' );
        while close_par = fail do
            function_head_line_number := function_head_line_number + 1;
            Append( function_head, list[ function_head_line_number ] );
            close_par := Position( function_head, ')' );
        od;
        
        arguments := SplitString( function_head{[ open_par + 1 .. close_par - 1 ]}, "," );
        
        arguments := List( arguments, i -> SplitString( NormalizedWhitespace( i ), " " ) );
        arguments := List( arguments, function( i ) if Length( i ) = 1 then return false; else return i[ 1 ]; fi; end );
        current_func_rec.arguments := arguments;
        
        current_function_line := current_function_line{[ Position( current_function_line, ',' ) + 1 .. Length( current_function_line ) ]};
        current_function_line := NormalizedWhitespace( current_function_line );
        current_function_line := current_function_line{[ Position( current_function_line, ' ' ) + 1 ..  Length( current_function_line ) ]};
        body_begin := Int( current_function_line{[ 1 .. Position( current_function_line, ',' ) - 1 ]} );
        
        doc_string := Concatenation( list{[ function_head_line_number + 1 .. body_begin - 1 ]} );
        current_func_rec.doc_string := doc_string;
        
    od;
    
    return output_rec;
    
end;

list_of_records := [ ];

for i in lib_files do
    record := extract_info_singular_lib( Concatenation( singular_dir, i ), location_libparse, i );
#     textfilename := i{[ 5 .. Length( i ) ]};
#     text_file := OutputTextFile( Concatenation( textfilename, ".docu" ), false );
#     GapToJsonStream( text_file, record );
#     CloseStream( text_file );
    Add( list_of_records, record );
od;

doc_string_types := [ "USAGE", "RETURN", "ASSUME", "NOTES", "SEE ALSO", "EXAMPLE" ];

for current_lib in list_of_records do
    Print( current_lib.library_filename, "\n" ); 
    for current_func in current_lib.funcs do
        current_doc_string := current_func.doc_string;
        current_func.doc_string := rec( );
        current_func.doc_string_plain := current_doc_string;
        positions := List( doc_string_types, i -> PositionSublist( current_doc_string, i ) );
        if DuplicateFreeList( positions ) = [ fail ] then
            current_func.unable_to_split_doc_string := true;
            continue;
        fi;
        Add( positions, Length( current_doc_string ) + 1 );
        current_doc_string_pos := FindNextIndexEntryInList( positions, 0 );
        i := current_doc_string_pos[ 1 ];
        repeat
            current_entry := doc_string_types[ current_doc_string_pos[ 2 ] ];
            current_doc_string_pos := FindNextIndexEntryInList( positions, i );
            current_func.doc_string.( current_entry ) := current_doc_string{[ i + Length( current_entry ) + 1 .. current_doc_string_pos[ 1 ] - 1 ]};
            i := current_doc_string_pos[ 1 ];
        until current_doc_string_pos[ 2 ] = Length( doc_string_types ) + 1;
    od;
od;

all_input_types_list_list := List( list_of_records, i -> List( i.funcs, j -> j.arguments ) );

all_input_types := [ ];

for i in all_input_types_list_list do
    for j in i do
        for k in j do
            Add( all_input_types, k );
        od;
    od;
od;

all_input_types := DuplicateFreeList( all_input_types );

Remove( all_input_types, Position( all_input_types, false ) );

for current_lib in list_of_records do
    for current_func in current_lib.funcs do
        if not IsBound( current_func.doc_string.RETURN ) then
            continue;
        fi;
        positions := List( all_input_types, i -> PositionSublist( current_func.doc_string.RETURN, i ) );
        current_minimum := Minimum( positions );
        if current_minimum = fail then
            continue;
        fi;
        minimum_pos := Position( positions, current_minimum );
        current_func.guessed_output_type := all_input_types[ minimum_pos ];
    od;
od;

text_file := OutputTextFile( "singularlibs.js", false );
GapToJsonStream( text_file, list_of_records );
CloseStream( text_file );



